/***************************************************************************
                          wepdlg.cpp  -  description
                             -------------------
    begin                : Tue Jun 27 2000
    copyright            : (C) 2000 by Jae-Jun Lee
    email                : brucejr@samsung.co.kr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "wepdlg.h"
#include "swlioctl.h"
#include "iwcommon.h"

/* MD5C.C - RSA Data Security, Inc., MD5 message-digest algorithm
 */

/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
rights reserved.

License to copy and use this software is granted provided that it
is identified as the "RSA Data Security, Inc. MD5 Message-Digest
Algorithm" in all material mentioning or referencing this software
or this function.

License is also granted to make and use derivative works provided
that such works are identified as "derived from the RSA Data
Security, Inc. MD5 Message-Digest Algorithm" in all material
mentioning or referencing the derived work.

RSA Data Security, Inc. makes no representations concerning either
the merchantability of this software or the suitability of this
software for any particular purpose. It is provided "as is"
without express or implied warranty of any kind.

These notices must be retained in any copies of any part of this
documentation and/or software.
 */

/* Constants for MD5Transform routine.
 */
#define S11 7
#define S12 12
#define S13 17
#define S14 22
#define S21 5
#define S22 9
#define S23 14
#define S24 20
#define S31 4
#define S32 11
#define S33 16
#define S34 23
#define S41 6
#define S42 10
#define S43 15
#define S44 21



static unsigned char PADDING[64] = {
  0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

/* F, G, H and I are basic MD5 functions.
 */
#define F(x, y, z) (((x) & (y)) | ((~x) & (z)))
#define G(x, y, z) (((x) & (z)) | ((y) & (~z)))
#define H(x, y, z) ((x) ^ (y) ^ (z))
#define I(x, y, z) ((y) ^ ((x) | (~z)))

/* ROTATE_LEFT rotates x left n bits.
 */
#define ROTATE_LEFT(x, n) (((x) << (n)) | ((x) >> (32-(n))))

/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
Rotation is separate from addition to prevent recomputation.
 */
#define FF(a, b, c, d, x, s, ac) { \
 (a) += F ((b), (c), (d)) + (x) + (DWORD)(ac); \
 (a) = ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }
#define GG(a, b, c, d, x, s, ac) { \
 (a) += G ((b), (c), (d)) + (x) + (DWORD)(ac); \
 (a) = ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }
#define HH(a, b, c, d, x, s, ac) { \
 (a) += H ((b), (c), (d)) + (x) + (DWORD)(ac); \
 (a) = ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }
#define II(a, b, c, d, x, s, ac) { \
 (a) += I ((b), (c), (d)) + (x) + (DWORD)(ac); \
 (a) = ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }


#define KEY_LENGTH  20   // 4 KEYS x 5 char(8bit) = 40bits x 4 Keys
#define KEY_NUM     4

// MD5 initialization. Begins an MD5 operation, writing a new context.
void wepDlg::MD5Init(MD5_CTX *context)
{
  context->count[0] = context->count[1] = 0;
  // Load magic initialization constants.

  context->state[0] = 0x67452301;
  context->state[1] = 0xefcdab89;
  context->state[2] = 0x98badcfe;
  context->state[3] = 0x10325476;

}

/* MD5 block update operation. Continues an MD5 message-digest
  operation, processing another message block, and updating the
  context.
 */
void wepDlg::MD5Update(MD5_CTX *context, unsigned char *input, unsigned char inputLen)
{

   unsigned int i, index, partLen;

  /* Compute number of bytes mod 64 */
   index = (unsigned int)((context->count[0] >> 3) & 0x3F);

  /* Update number of bits */
  if ((context->count[0] += ((DWORD)inputLen << 3))
   < ((DWORD)inputLen << 3))
  context->count[1]++;
  context->count[1] += ((DWORD)inputLen >> 29);

  partLen = 64 - index;

  /* Transform as many times as possible.
 */
  if (inputLen >= partLen) {
  MD5_memcpy
   ((BYTE*)&context->buffer[index], (BYTE*)input, partLen);
  MD5Transform (context->state, context->buffer);

  for (i = partLen; i + 63 < inputLen; i += 64)
   MD5Transform (context->state, &input[i]);

  index = 0;
  }
  else
  i = 0;

  /* Buffer remaining input */
  MD5_memcpy
  ((BYTE*)&context->buffer[index], (BYTE*)&input[i],
  inputLen-i);
}

/* MD5 finalization. Ends an MD5 message-digest operation, writing the
  the message digest and zeroizing the context.
 */
void wepDlg::MD5Final(unsigned char *digest, MD5_CTX *context)
{
  unsigned char bits[8];
  unsigned int index, padLen;

  /* Save number of bits */
  Encode (bits, context->count, 8);

  /* Pad out to 56 mod 64.
*/
  index = (unsigned int)((context->count[0] >> 3) & 0x3f);
  padLen = (index < 56) ? (56 - index) : (120 - index);
  MD5Update (context, PADDING, padLen);

  /* Append length (before padding) */
  MD5Update (context, bits, 8);

  /* Store state in digest */
  Encode (digest, context->state, 16);

  /* Zeroize sensitive information.
*/
  MD5_memset ((BYTE*)context, 0, sizeof (*context));
}

/* MD5 basic transformation. Transforms state based on block.
 */
void wepDlg::MD5Transform(DWORD *state, unsigned char *block)
{
  DWORD a = state[0], b = state[1], c = state[2], d = state[3], x[16];

  Decode (x, block, 64);

  /* Round 1 */
  FF (a, b, c, d, x[ 0], S11, 0xd76aa478); /* 1 */
  FF (d, a, b, c, x[ 1], S12, 0xe8c7b756); /* 2 */
  FF (c, d, a, b, x[ 2], S13, 0x242070db); /* 3 */
  FF (b, c, d, a, x[ 3], S14, 0xc1bdceee); /* 4 */
  FF (a, b, c, d, x[ 4], S11, 0xf57c0faf); /* 5 */
  FF (d, a, b, c, x[ 5], S12, 0x4787c62a); /* 6 */
  FF (c, d, a, b, x[ 6], S13, 0xa8304613); /* 7 */
  FF (b, c, d, a, x[ 7], S14, 0xfd469501); /* 8 */
  FF (a, b, c, d, x[ 8], S11, 0x698098d8); /* 9 */
  FF (d, a, b, c, x[ 9], S12, 0x8b44f7af); /* 10 */
  FF (c, d, a, b, x[10], S13, 0xffff5bb1); /* 11 */
  FF (b, c, d, a, x[11], S14, 0x895cd7be); /* 12 */
  FF (a, b, c, d, x[12], S11, 0x6b901122); /* 13 */
  FF (d, a, b, c, x[13], S12, 0xfd987193); /* 14 */
  FF (c, d, a, b, x[14], S13, 0xa679438e); /* 15 */
  FF (b, c, d, a, x[15], S14, 0x49b40821); /* 16 */

 /* Round 2 */
  GG (a, b, c, d, x[ 1], S21, 0xf61e2562); /* 17 */
  GG (d, a, b, c, x[ 6], S22, 0xc040b340); /* 18 */
  GG (c, d, a, b, x[11], S23, 0x265e5a51); /* 19 */
  GG (b, c, d, a, x[ 0], S24, 0xe9b6c7aa); /* 20 */
  GG (a, b, c, d, x[ 5], S21, 0xd62f105d); /* 21 */
  GG (d, a, b, c, x[10], S22,  0x2441453); /* 22 */
  GG (c, d, a, b, x[15], S23, 0xd8a1e681); /* 23 */
  GG (b, c, d, a, x[ 4], S24, 0xe7d3fbc8); /* 24 */
  GG (a, b, c, d, x[ 9], S21, 0x21e1cde6); /* 25 */
  GG (d, a, b, c, x[14], S22, 0xc33707d6); /* 26 */
  GG (c, d, a, b, x[ 3], S23, 0xf4d50d87); /* 27 */
  GG (b, c, d, a, x[ 8], S24, 0x455a14ed); /* 28 */
  GG (a, b, c, d, x[13], S21, 0xa9e3e905); /* 29 */
  GG (d, a, b, c, x[ 2], S22, 0xfcefa3f8); /* 30 */
  GG (c, d, a, b, x[ 7], S23, 0x676f02d9); /* 31 */
  GG (b, c, d, a, x[12], S24, 0x8d2a4c8a); /* 32 */

  /* Round 3 */
  HH (a, b, c, d, x[ 5], S31, 0xfffa3942); /* 33 */
  HH (d, a, b, c, x[ 8], S32, 0x8771f681); /* 34 */
  HH (c, d, a, b, x[11], S33, 0x6d9d6122); /* 35 */
  HH (b, c, d, a, x[14], S34, 0xfde5380c); /* 36 */
  HH (a, b, c, d, x[ 1], S31, 0xa4beea44); /* 37 */
  HH (d, a, b, c, x[ 4], S32, 0x4bdecfa9); /* 38 */
  HH (c, d, a, b, x[ 7], S33, 0xf6bb4b60); /* 39 */
  HH (b, c, d, a, x[10], S34, 0xbebfbc70); /* 40 */
  HH (a, b, c, d, x[13], S31, 0x289b7ec6); /* 41 */
  HH (d, a, b, c, x[ 0], S32, 0xeaa127fa); /* 42 */
  HH (c, d, a, b, x[ 3], S33, 0xd4ef3085); /* 43 */
  HH (b, c, d, a, x[ 6], S34,  0x4881d05); /* 44 */
  HH (a, b, c, d, x[ 9], S31, 0xd9d4d039); /* 45 */
  HH (d, a, b, c, x[12], S32, 0xe6db99e5); /* 46 */
  HH (c, d, a, b, x[15], S33, 0x1fa27cf8); /* 47 */
  HH (b, c, d, a, x[ 2], S34, 0xc4ac5665); /* 48 */

  /* Round 4 */
  II (a, b, c, d, x[ 0], S41, 0xf4292244); /* 49 */
  II (d, a, b, c, x[ 7], S42, 0x432aff97); /* 50 */
  II (c, d, a, b, x[14], S43, 0xab9423a7); /* 51 */
  II (b, c, d, a, x[ 5], S44, 0xfc93a039); /* 52 */
  II (a, b, c, d, x[12], S41, 0x655b59c3); /* 53 */
  II (d, a, b, c, x[ 3], S42, 0x8f0ccc92); /* 54 */
  II (c, d, a, b, x[10], S43, 0xffeff47d); /* 55 */
  II (b, c, d, a, x[ 1], S44, 0x85845dd1); /* 56 */
  II (a, b, c, d, x[ 8], S41, 0x6fa87e4f); /* 57 */
  II (d, a, b, c, x[15], S42, 0xfe2ce6e0); /* 58 */
  II (c, d, a, b, x[ 6], S43, 0xa3014314); /* 59 */
  II (b, c, d, a, x[13], S44, 0x4e0811a1); /* 60 */
  II (a, b, c, d, x[ 4], S41, 0xf7537e82); /* 61 */
  II (d, a, b, c, x[11], S42, 0xbd3af235); /* 62 */
  II (c, d, a, b, x[ 2], S43, 0x2ad7d2bb); /* 63 */
  II (b, c, d, a, x[ 9], S44, 0xeb86d391); /* 64 */

  state[0] += a;
  state[1] += b;
  state[2] += c;
  state[3] += d;

  /* Zeroize sensitive information.
*/
  MD5_memset ((BYTE*)x, 0, sizeof (x));
}

/* Encodes input (DWORD) into output (unsigned char). Assumes len is
  a multiple of 4.
 */
void wepDlg::Encode (unsigned char *output, DWORD* input, unsigned int len)
{
  unsigned int i, j;

  for (i = 0, j = 0; j < len; i++, j += 4) {
 output[j] = (unsigned char)(input[i] & 0xff);
 output[j+1] = (unsigned char)((input[i] >> 8) & 0xff);
 output[j+2] = (unsigned char)((input[i] >> 16) & 0xff);
 output[j+3] = (unsigned char)((input[i] >> 24) & 0xff);
  }
}

/* Decodes input (unsigned char) into output (DWORD). Assumes len is
  a multiple of 4.
 */
void wepDlg::Decode (DWORD *output,unsigned char *input,unsigned int len)
{
  unsigned int i, j;

  for (i = 0, j = 0; j < len; i++, j += 4)
 output[i] = ((DWORD)input[j]) | (((DWORD)input[j+1]) << 8) |
   (((DWORD)input[j+2]) << 16) | (((DWORD)input[j+3]) << 24);
}

/* Note: Replace "for loop" with standard memcpy if possible.
 */

void wepDlg::MD5_memcpy (BYTE* output,BYTE* input,unsigned int len)
{
  unsigned int i;

  for (i = 0; i < len; i++)
 output[i] = input[i];
}

/* Note: Replace "for loop" with standard memset if possible.
 */
void wepDlg::MD5_memset (BYTE* output, int value,unsigned int len)
{
  unsigned int i;

  for (i = 0; i < len; i++)
 ((char *)output)[i] = (char)value;
}
//=========================== MD5 END =======================================================

wepDlg::wepDlg(QWidget *parent, const char *name) : QDialog(parent,name,true){
	initDialog();
}

wepDlg::~wepDlg(){
}

void wepDlg::setDialogWindows(char* name,int skfd1,int wepreq1){
   struct iwreq wrq;
   char buf[256],*bp,hex[10],charKey[50];
   QString strTemp;
   int i;

   bp = buf;

   wepreq = wepreq1;
   strcpy(ifname,name); //adaptr name,eth0...
   skfd = skfd1;        //socket pointer.
   strcpy(wrq.ifr_name,ifname);

   defaultKeyID->insertItem("Key 1");
   if(wepreq != 7){
	defaultKeyID->insertItem("Key 2");
	defaultKeyID->insertItem("Key 3");
	defaultKeyID->insertItem("Key 4");
   }
   //get default wep key ID(SWL_UIL_WEPKEYID_GET).
   //WEP
   wrq.u.data.pointer = (caddr_t)buf;
   wrq.u.data.length = 1;
   wrq.u.data.flags = SWL_UIL_WEPKEYID_GET;
   if(ioctl(skfd,SIOCDEVPRIVATE , &wrq) < 0)
    {
       debug("There is a problem while reading WEP Default Key ID ...\n");
       defaultKeyID->setCurrentItem(0);
     }
   else{
       defaultKeyID->setCurrentItem(buf[0]);
   }

   Key1->clear();
   Key2->clear();
   Key3->clear();
   Key4->clear();
   //read key1
   wrq.u.data.pointer = (caddr_t)buf;
   wrq.u.data.length = 1;
   wrq.u.data.flags = SWL_UIL_WEPKEY1_GET;
   if(ioctl(skfd,SIOCDEVPRIVATE , &wrq) < 0)
    {
       debug("There is a problem while reading WEP Default Key ID ...\n");
     }
   else{
      if(wepreq == 7){
        for(i=0;i<13;++i){
           if((buf[i] & 0xf0) != 0)
               sprintf(hex,"%x",buf[i] & 0xFF);
           else
              sprintf(hex,"0%x",buf[i] & 0xFF);

           charKey[2*i]   = hex[0];
           charKey[2*i+1] = hex[1];
         }

        charKey[26] = '\0';
        Key1->setText((const char*)charKey);
      }
      else{
	  // debug("key[%d] = %x%x%x%x%x\n",0,buf[0] & 0xff,buf[1]& 0xFF,buf[2]& 0xFF,
  	  //							buf[3]&0xFF,buf[4]&0xFF);
        for(i=0;i<5;++i){
         if((buf[i] & 0xf0) != 0)
           sprintf(hex,"%x",buf[i] & 0xFF);
         else
           sprintf(hex,"0%x",buf[i] & 0xFF);

         charKey[2*i]   = hex[0];
         charKey[2*i+1] = hex[1];
      }
      charKey[10] = '\0';
      Key1->setText((const char*)charKey);
      }

   }

   if(wepreq != 7){
     wrq.u.data.pointer = (caddr_t)buf;
     wrq.u.data.length = 1;
     wrq.u.data.flags = SWL_UIL_WEPKEY2_GET;
     if(ioctl(skfd,SIOCDEVPRIVATE , &wrq) < 0)
      {
         debug("There is a problem while reading WEP Default Key ID ...\n");
       }
     else{
      for(i=0;i<5;++i){
         if((buf[i] & 0xf0) != 0)
           sprintf(hex,"%x",buf[i] & 0xFF);
         else
           sprintf(hex,"0%x",buf[i] & 0xFF);

         charKey[2*i]   = hex[0];
         charKey[2*i+1] = hex[1];
      }
      charKey[10] = '\0';
      Key2->setText((const char*)charKey);
   }

   wrq.u.data.pointer = (caddr_t)buf;
   wrq.u.data.length = 1;
   wrq.u.data.flags = SWL_UIL_WEPKEY3_GET;
   if(ioctl(skfd,SIOCDEVPRIVATE , &wrq) < 0)
    {
       debug("There is a problem while reading WEP Default Key ID ...\n");
     }
   else{
      for(i=0;i<5;++i){
         if((buf[i] & 0xf0) != 0)
           sprintf(hex,"%x",buf[i] & 0xFF);
         else
           sprintf(hex,"0%x",buf[i] & 0xFF);

         charKey[2*i]   = hex[0];
         charKey[2*i+1] = hex[1];
      }
      charKey[10] = '\0';
      Key3->setText((const char*)charKey);
   }

   wrq.u.data.pointer = (caddr_t)buf;
   wrq.u.data.length = 1;
   wrq.u.data.flags = SWL_UIL_WEPKEY4_GET;
   if(ioctl(skfd,SIOCDEVPRIVATE , &wrq) < 0)
    {
       debug("There is a problem while reading WEP Default Key ID ...\n");
     }
   else{
      for(i=0;i<5;++i){
         if((buf[i] & 0xf0) != 0)
           sprintf(hex,"%x",buf[i] & 0xFF);
         else
           sprintf(hex,"0%x",buf[i] & 0xFF);

         charKey[2*i]   = hex[0];
         charKey[2*i+1] = hex[1];
      }
      charKey[10] = '\0';
      Key4->setText((const char*)charKey);
   }
  }
   setCaption("WEP KEY Setup");

   if(wepreq == 7){
     Key2->setEnabled(false);
     Key3->setEnabled(false);
     Key4->setEnabled(false);
     defaultKeyID->setCurrentItem(0);
   }

   passphrase->setFocus();
   QObject::connect(OKButton,SIGNAL(clicked()),this,SLOT(accept()));
   QObject::connect(WriteButton,SIGNAL(clicked()),this,SLOT(OnOK()));
   QObject::connect(GenerateButton,SIGNAL(clicked()),this,SLOT(OnGenerate()));
   QObject::connect(ClearButton,SIGNAL(clicked()),this,SLOT(OnClear()));
   QObject::connect(passphrase,SIGNAL(returnPressed()),this,SLOT(OnGenerate()));

}/**  */
void wepDlg::OnOK(){
   struct iwreq wrq;
   char buf[256],*bp;
   //const char constBuf[10],*c_bp;
   int temp,i;
   QString strTemp;

   bp = buf;
   //c_bp = constBuf;
   strcpy(wrq.ifr_name, ifname);

   buf[0] = wepreq;
   wrq.u.data.pointer = (caddr_t)buf;
   wrq.u.data.length = 1;
   wrq.u.data.flags = SWL_UIL_WEPFLAGS_SET;
   if(ioctl(skfd,SIOCDEVPRIVATE , &wrq) < 0)
   {
       debug("There is a problem while setting WEP...\n");
     }

   //WEP KEY
   strTemp = Key1->text();
   strcpy(buf,strTemp);
   //debug("key1=%s\n",buf);
   wrq.u.data.pointer = (caddr_t)buf;
   wrq.u.data.length = strlen(buf);
   wrq.u.data.flags = SWL_UIL_WEPKEY1_SET;
   if(ioctl(skfd, SIOCDEVPRIVATE, &wrq) < 0)
    {
       debug("There is a problem while setting WEP KEY 1\n");
    }

    strTemp = Key2->text();
    if(wepreq == 7)
     for(i=0;i<10;++i)
       buf[i] ='0';
    else
     strcpy(buf,strTemp);

    //debug("key2=%s\n",buf);
    wrq.u.data.pointer = (caddr_t)buf;
    wrq.u.data.length = strlen(buf);
    wrq.u.data.flags = SWL_UIL_WEPKEY2_SET;
    if(ioctl(skfd, SIOCDEVPRIVATE, &wrq) < 0)
     {
       debug("There is a problem while setting WEP KEY 2\n");
     }

    strTemp = Key3->text();
    if(wepreq == 7)
     for(i=0;i<10;++i)
       buf[i] ='0';
    else
       strcpy(buf,strTemp);
    //debug("key3=%s\n",buf);
    wrq.u.data.pointer = (caddr_t)buf;
    wrq.u.data.length = strlen(buf);
    wrq.u.data.flags = SWL_UIL_WEPKEY3_SET;
    if(ioctl(skfd, SIOCDEVPRIVATE, &wrq) < 0)
     {
        debug("There is a problem while setting WEP KEY 3\n");
     }
    strTemp = Key4->text();
	if(wepreq == 7)
     for(i=0;i<10;++i)
       buf[i] ='0';
    else
       strcpy(buf,strTemp);
    //debug("key4=%s\n",buf);
    wrq.u.data.pointer = (caddr_t)buf;
    wrq.u.data.length = strlen(buf);
    wrq.u.data.flags = SWL_UIL_WEPKEY4_SET;
    if(ioctl(skfd, SIOCDEVPRIVATE, &wrq) < 0)
     {
        debug("There is a problem while setting WEP KEY 4\n");
     }

   //Default WEP KEY ID.
    buf[0] = defaultKeyID->currentItem();
    wrq.u.data.pointer = (caddr_t)buf;
    wrq.u.data.length = 1;
    wrq.u.data.flags = SWL_UIL_WEPKEYID_SET;
    if(ioctl(skfd, SIOCDEVPRIVATE, &wrq) < 0)
    {
      debug("There is a problem while seting WEP Default Key ID\n");
    }
    //software reset.
   wrq.u.data.pointer = (caddr_t)buf;
   wrq.u.data.length = 1;
   wrq.u.data.flags = SWL_UIL_SOFTRESET;
   if(ioctl(skfd,SIOCDEVPRIVATE, &wrq) < 0)
    {
       debug("There is a problem while SOFT RESET\n");
    }

 // QDialog::accept();
}
/**  */
void wepDlg::OnGenerate(){
  int len,i;
  char buf[256],hex[10],charKey[50];
  QString strTemp;

  strTemp = passphrase->text();
  strcpy(buf,strTemp);

  //debug("input text(%d)=%s",strlen(buf),buf);


  if(wepreq == 7){
    keyGenerate128(strlen(buf),buf,key);
   // debug("key[%d] = %x%x%x%x%x%x%x%x%x%x%x%x%x\n",0,key[0] & 0xff,key[1] & 0xff,key[2] & 0xff,
   //		key[3] & 0xff,key[4] & 0xff,key[5] & 0xff,key[6] & 0xff,key[7] & 0xff,
   //		key[8] & 0xff,key[9] & 0xff,key[10] & 0xff,key[11] & 0xff,key[12] & 0xff);

   for(i=0;i<13;++i){
     if((key[i] & 0xf0) != 0)
         sprintf(hex,"%x",key[i] & 0xFF);
     else
         sprintf(hex,"0%x",key[i] & 0xFF);

     charKey[2*i]   = hex[0];
     charKey[2*i+1] = hex[1];
   }

   charKey[26] = '\0';
   Key1->setText((const char*)charKey);
  								
  }
  else{
    keyGenerate(buf,wep_key);
    //debug("key[%d] = %x%x%x%x%x\n",0,wep_key[0][0] & 0xff,wep_key[0][1]& 0xFF,wep_key[0][2]& 0xFF,
    //								wep_key[0][3]&0xFF,wep_key[0][4]&0xFF);
    //debug("key[%d] = %x%x%x%x%x\n",1,wep_key[1][0] & 0xff,wep_key[1][1]& 0xFF,wep_key[1][2]& 0xFF,
    //								wep_key[1][3]&0xFF,wep_key[1][4]&0xFF);
    //debug("key[%d] = %x%x%x%x%x\n",2,wep_key[2][0] & 0xff,wep_key[2][1]& 0xFF,wep_key[2][2]& 0xFF,
    //								wep_key[2][3]&0xFF,wep_key[2][4]&0xFF);
    //debug("key[%d] = %x%x%x%x%x\n",3,wep_key[3][0] & 0xff,wep_key[3][1]& 0xFF,wep_key[3][2]& 0xFF,
    //								wep_key[3][3]&0xFF,wep_key[3][4]&0xFF);

    for(i=0;i<5;++i){
     if((wep_key[0][i] & 0xf0) != 0)
         sprintf(hex,"%x",wep_key[0][i] & 0xFF);
     else
         sprintf(hex,"0%x",wep_key[0][i] & 0xFF);

     charKey[2*i]   = hex[0];
     charKey[2*i+1] = hex[1];
   }
   charKey[10] = '\0';
   Key1->setText((const char*)charKey);

   for(i=0;i<5;++i){
     if((wep_key[1][i] & 0xf0) != 0)
         sprintf(hex,"%x",wep_key[1][i] & 0xFF);
     else
         sprintf(hex,"0%x",wep_key[1][i] & 0xFF);

     charKey[2*i]   = hex[0];
     charKey[2*i+1] = hex[1];
   }
   charKey[10] = '\0';
   Key2->setText((const char*)charKey);

   for(i=0;i<5;++i){
     if((wep_key[2][i] & 0xf0) != 0)
         sprintf(hex,"%x",wep_key[2][i] & 0xFF);
     else
         sprintf(hex,"0%x",wep_key[2][i] & 0xFF);

     charKey[2*i]   = hex[0];
     charKey[2*i+1] = hex[1];
   }
   charKey[10] = '\0';
   Key3->setText((const char*)charKey);

   for(i=0;i<5;++i){
     if((wep_key[3][i] & 0xf0) != 0)
         sprintf(hex,"%x",wep_key[3][i] & 0xFF);
     else
         sprintf(hex,"0%x",wep_key[3][i] & 0xFF);

     charKey[2*i]   = hex[0];
     charKey[2*i+1] = hex[1];
   }
   charKey[10] = '\0';
   Key4->setText((const char*)charKey);

   }

}
/**  */
void wepDlg::keyGenerate(char *genstr, unsigned char wep_key[4][5]){
   	unsigned int i,j;
	unsigned char pseed[4]={0,0,0,0};
	unsigned int len;
	int randNumber=0;

	len = strlen(genstr);
	if (len) {
		/* generate seed for random number generator using */
		/* key string... */
		for (i=0; i<len; i++) {
			pseed[i%4]^= genstr[i];
		}

		/* init PRN generator... note that this is equivalent */
		/*  to the Microsoft srand() function. */
		randNumber =	(int)pseed[0] |
				((int)pseed[1])<<8 |
				((int)pseed[2])<<16 |
				((int)pseed[3])<<24;

		/* generate keys. */
		for (i=0; i<4; i++) {
			for (j=0; j<5; j++) {
				/* Note that these three lines are */
				/* equivalent to the Microsoft rand() */
				/* function. */
				randNumber *= 0x343fd;
				randNumber += 0x269ec3;
				wep_key[i][j] = (unsigned char)((randNumber>>16) & 0x7fff);
			}

		}
	}
	return;
}
/**  */
void wepDlg::keyGenerate128(int len, char* m_Key_string,unsigned char* key){
  // assume 104-bit key and use MD5 for passphrase munging
				MD5_CTX		MD;
				char*		cp;
				char		password_buf[65];
				int			i;
				//UCHAR		key1[16];

				// Initialize MD5 structures
				MD5Init(&MD);

				// concatenate input passphrase repeatedly to fill password_buf
				cp = password_buf;
				for (i=0; i<64; i++)
					*cp++ = m_Key_string[i % len];

				// generate 128-bit signature using MD5
				MD5Update(&MD, (unsigned char *)password_buf, 64);
				MD5Final(key, &MD);

				//	copy 13 bytes (104 bits) of MG5 generated signature
}

/**  */
void wepDlg::OnClear(){
   if(wepreq ==7)
     Key1->setText("000000000000000000000000");
   else
     Key1->setText("00000000");

   Key2->setText("00000000");
   Key3->setText("00000000");
   Key4->setText("00000000");

   passphrase->clear();
}
